const logger = require('@pubsweet/logger')
const get = require('lodash/get')
const helpers = require('../../helpers/helpers')
const collectionHelper = require('../../helpers/Collection')
const teamHelper = require('../../helpers/Team')
const config = require('config')
const mailService = require('pubsweet-component-mail-service')
const userHelper = require('../../helpers/User')
const invitationHelper = require('../../helpers/Invitation')
const authsomeHelper = require('../../helpers/authsome')

const configRoles = config.get('roles')

module.exports = models => async (req, res) => {
  const { email, role } = req.body

  if (!helpers.checkForUndefinedParams(email, role)) {
    res.status(400).json({ error: 'Email and role are required' })
    logger.error('User ID and role are missing')
    return
  }

  if (!configRoles.collection.includes(role)) {
    res.status(400).json({ error: `Role ${role} is invalid` })
    logger.error(`invitation attempted on invalid role ${role}`)
    return
  }
  const reqUser = await models.User.find(req.user)

  if (email === reqUser.email && !reqUser.admin) {
    logger.error(`${reqUser.email} tried to invite his own email`)
    return res.status(400).json({ error: 'Cannot invite yourself' })
  }

  const collectionId = get(req, 'params.collectionId')
  let collection
  try {
    collection = await models.Collection.find(collectionId)
  } catch (e) {
    const notFoundError = await helpers.handleNotFoundError(e, 'collection')
    return res.status(notFoundError.status).json({
      error: notFoundError.message,
    })
  }

  const authsome = authsomeHelper.getAuthsome(models)
  const target = {
    collection,
    path: req.route.path,
  }
  const canPost = await authsome.can(req.user, 'POST', target)
  if (!canPost)
    return res.status(403).json({
      error: 'Unauthorized.',
    })
  const baseUrl = `${req.protocol}://${req.get('host')}`
  const params = {
    baseUrl,
    FragmentModel: models.Fragment,
    UserModel: models.User,
    collection,
    mailService,
    resend: false,
  }

  try {
    const user = await models.User.findByEmail(email)

    await teamHelper.setupManuscriptTeam(models, user, collectionId, role)
    let invitation = invitationHelper.getInvitation(
      collection.invitations,
      user.id,
      role,
    )

    let resend = false
    if (invitation !== undefined) {
      if (invitation.hasAnswer)
        return res
          .status(400)
          .json({ error: 'User has already replied to a previous invitation.' })
      invitation.invitedOn = Date.now()
      await collection.save()
      resend = true
    } else {
      invitation = await invitationHelper.setupInvitation(
        user.id,
        role,
        collection,
      )
    }

    try {
      if (role === 'reviewer') {
        if (collection.status === 'heAssigned')
          await collectionHelper.updateStatus(collection, 'reviewersInvited')

        await invitationHelper.setupReviewerInvitation({
          ...params,
          user,
          invitationId: invitation.id,
          timestamp: invitation.invitedOn,
          resend,
        })
      }

      if (role === 'handlingEditor') {
        invitation.invitedOn = Date.now()
        await collection.save()
        await collectionHelper.addHandlingEditor(collection, user, invitation)
        await mailService.setupAssignEmail(
          user.email,
          'assign-handling-editor',
          baseUrl,
        )
      }
      return res.status(200).json(invitation)
    } catch (e) {
      logger.error(e)
      return res.status(500).json({ error: 'Email could not be sent.' })
    }
  } catch (e) {
    if (role === 'reviewer') {
      const newUser = await userHelper.setupNewUser(
        req.body,
        baseUrl,
        res,
        email,
        role,
        models.User,
        'invite',
      )
      if (newUser.error !== undefined) {
        return res.status(newUser.status).json({
          error: newUser.message,
        })
      }
      if (collection.status === 'heAssigned')
        await collectionHelper.updateStatus(collection, 'reviewersInvited')
      await teamHelper.setupManuscriptTeam(models, newUser, collectionId, role)
      const invitation = await invitationHelper.setupInvitation(
        newUser.id,
        role,
        collection,
      )

      await invitationHelper.setupReviewerInvitation({
        ...params,
        user: newUser,
        invitationId: invitation.id,
        timestamp: invitation.invitedOn,
      })
      return res.status(200).json(invitation)
    }
    const notFoundError = await helpers.handleNotFoundError(e, 'user')
    return res.status(notFoundError.status).json({
      error: notFoundError.message,
    })
  }
}
