import React, { Fragment } from 'react'
import { isEmpty } from 'lodash'
import { connect } from 'react-redux'
import { required } from 'xpub-validators'
import { withJournal } from 'xpub-journal'
import styled, { css } from 'styled-components'
import {
  th,
  Menu,
  ValidatedField,
  Icon,
  Button,
  ErrorText,
  Spinner,
} from '@pubsweet/ui'
import { compose, withHandlers, withProps } from 'recompose'
import {
  reduxForm,
  isSubmitting,
  change as changeForm,
  getFormValues,
} from 'redux-form'
import AutosaveIndicator from 'pubsweet-component-wizard/src/components/AutosaveIndicator'

import {
  uploadFile,
  deleteFile,
  getSignedUrl,
  getRequestStatus,
  getFileError,
} from 'pubsweet-components-faraday/src/redux/files'
import {
  FileItem,
  FilePicker,
} from 'pubsweet-components-faraday/src/components/Files'

import {
  ConfirmationModal,
  withModal2,
} from 'pubsweet-component-modal/src/components'
import {
  selectError,
  selectFetching,
  createRecommendation,
  updateRecommendation,
} from 'pubsweet-components-faraday/src/redux/recommendations'

import {
  parseReviewResponseToForm,
  onReviewSubmit,
  onReviewChange,
} from './utils'

const guidelinesLink =
  'https://about.hindawi.com/authors/peer-review-at-hindawi/'

const ReviewerReportForm = ({
  addFile,
  removeFile,
  review = {},
  errorRequest,
  isSubmitting,
  changeField,
  handleSubmit,
  formValues = {},
  fileFetching,
  fileError,
  journal: { recommendations },
}) => (
  <Root>
    <Row>
      <Label>Recommendation*</Label>
      <ActionLink href={guidelinesLink} target="_blank">
        Hindawi Reviewer Guidelines
      </ActionLink>
    </Row>
    <Row>
      <ValidatedField
        component={input => (
          <Menu
            {...input}
            inline
            onChange={v => changeField('recommendation', v)}
            options={recommendations}
            placeholder="Select"
          />
        )}
        name="recommendation"
        validate={[required]}
      />
    </Row>
    <Spacing />
    <Row left>
      <Label>Report</Label>
      {!fileFetching.review ? (
        <FilePicker
          allowedFileExtensions={['pdf', 'doc', 'docx']}
          onUpload={addFile}
        >
          <ActionText left={12}>Upload file</ActionText>
        </FilePicker>
      ) : (
        <Spinner size={2} />
      )}
    </Row>
    <Row>
      <FullWidth>
        <ValidatedField
          component={input => (
            <Textarea
              {...input}
              hasError={input.validationStatus === 'error'}
              onChange={e => changeField('public', e.target.value)}
              rows={6}
            />
          )}
          name="public"
          validate={isEmpty(formValues.files) ? [required] : []}
        />
      </FullWidth>
    </Row>
    {formValues.files && (
      <Row left>
        {formValues.files.map(file => (
          <FileItem
            compact
            id={file.id}
            key={file.id}
            {...file}
            removeFile={removeFile}
          />
        ))}
      </Row>
    )}
    {formValues.hasConfidential ? (
      <Fragment>
        <Row>
          <Label>
            Note for the editorial team <i>Not shared with the author</i>
          </Label>
          <ActionTextIcon onClick={() => changeField('hasConfidential', false)}>
            <Icon primary size={3}>
              x
            </Icon>
            Remove
          </ActionTextIcon>
        </Row>
        <Row>
          <FullWidth>
            <ValidatedField
              component={input => (
                <Textarea
                  {...input}
                  hasError={input.validationStatus === 'error'}
                  onChange={e => changeField('confidential', e.target.value)}
                  rows={6}
                />
              )}
              name="confidential"
              validate={[required]}
            />
          </FullWidth>
        </Row>
      </Fragment>
    ) : (
      <Row>
        <ActionText onClick={() => changeField('hasConfidential', true)}>
          Add confidential note for the Editorial Team
        </ActionText>
      </Row>
    )}

    <Spacing />
    {fileError && (
      <Row>
        <ErrorText>{fileError}</ErrorText>
      </Row>
    )}
    {errorRequest && (
      <Row>
        <ErrorText>{errorRequest}</ErrorText>
      </Row>
    )}
    <Row>
      <ActionButton onClick={handleSubmit}> Submit report </ActionButton>
      <AutosaveIndicator
        formName="reviewerReport"
        lastUpdated={review.updatedOn}
      />
    </Row>
  </Root>
)

const ModalWrapper = compose(
  connect(state => ({
    fetching: selectFetching(state),
    modalError: selectError(state),
  })),
)(({ fetching, ...rest }) => (
  <ConfirmationModal {...rest} isFetching={fetching} />
))

export default compose(
  withJournal,
  connect(
    state => ({
      formValues: getFormValues('reviewerReport')(state),
      isSubmitting: isSubmitting('reviewerReport')(state),
      errorRequest: selectError(state),
      fileFetching: getRequestStatus(state),
      fileError: getFileError(state),
    }),
    {
      changeForm,
      getFormValues,
      createRecommendation,
      updateRecommendation,
      uploadFile,
      deleteFile,
      getSignedUrl,
    },
  ),
  withProps(({ review = {} }) => ({
    initialValues: parseReviewResponseToForm(review),
  })),
  withModal2(props => ({
    modalComponent: ModalWrapper,
  })),
  withHandlers({
    changeField: ({ changeForm }) => (field, value) => {
      changeForm('reviewerReport', field, value)
    },
    addFile: ({ formValues = {}, uploadFile, changeForm, version }) => file => {
      uploadFile(file, 'review', version.id)
        .then(file => {
          const files = formValues.files || []
          const newFiles = [...files, file]

          setTimeout(() => {
            changeForm('reviewerReport', 'files', newFiles)
          }, 1000)
        })
        .catch(e => console.error(`Couldn't upload file.`, e))
    },
    removeFile: ({
      formValues: { files = [] },
      changeForm,
      deleteFile,
    }) => id => e => {
      deleteFile(id)
        .then(r => {
          const newFiles = files.filter(f => f.id !== id)
          changeForm('reviewerReport', 'files', newFiles)
        })
        .catch(e => console.error(`Couldn't delete the file.`, e))

      const newFiles = files.filter(f => f.id !== id)
      changeForm('reviewerReport', 'files', newFiles)
    },
  }),
  reduxForm({
    form: 'reviewerReport',
    enableReinitialize: true,
    keepDirtyOnReinitialize: true,
    onChange: onReviewChange,
    onSubmit: onReviewSubmit,
  }),
)(ReviewerReportForm)

// #region styled-components

const defaultText = css`
  color: ${th('colorPrimary')};
  font-family: ${th('fontReading')};
  font-size: ${th('fontSizeBaseSmall')};
`
const Root = styled.div`
  display: flex;
  flex-direction: column;
  margin: auto;
  [role='listbox'] {
    min-width: 280px;
  }
`

const Label = styled.div`
  ${defaultText};
  text-transform: uppercase;
  i {
    text-transform: none;
    margin-left: ${th('gridUnit')};
  }
`

const ActionText = styled.span`
  ${defaultText};
  text-decoration: underline;
  cursor: pointer;
  margin-left: ${({ left }) => left || 0}px;
`

const ActionTextIcon = styled(ActionText)`
  display: flex;
  align-items: center;
`
const ActionLink = styled.a`
  ${defaultText};
`

const Textarea = styled.textarea`
  width: 100%;
  padding: calc(${th('subGridUnit')}*2);
  font-size: ${th('fontSizeBaseSmall')};
  font-family: ${th('fontWriting')};
  border-color: ${({ hasError }) =>
    hasError ? th('colorError') : th('colorPrimary')};
`

const Spacing = styled.div`
  margin-top: ${th('gridUnit')};
  flex: 1;
`

const FullWidth = styled.div`
  flex: 1;
  > div {
    flex: 1;
  }
`

const Row = styled.div`
  display: flex;
  flex-direction: row;
  align-items: center;
  flex: 1;
  box-sizing: border-box;
  flex-wrap: wrap;
  justify-content: ${({ left }) => (left ? 'left' : 'space-between')};
  ${defaultText};
`

const ActionButton = styled(Button)`
  ${defaultText};
  align-items: center;
  background-color: ${th('colorPrimary')};
  color: ${th('colorTextReverse')};
  display: flex;
  padding: 4px 8px;
  text-align: center;
  height: calc(${th('subGridUnit')}*5);
  text-transform: uppercase;
`
// #endregion
