import React, { Fragment } from 'react'
import { pick } from 'lodash'
import { connect } from 'react-redux'
import { Icon, Checkbox } from '@pubsweet/ui'
import styled, { css } from 'styled-components'
import { compose, withHandlers, withProps } from 'recompose'
import { reduxForm, Field, change as changeForm } from 'redux-form'

import { Spinner } from '../UIComponents'

import {
  getAuthors,
  editAuthor,
  getAuthorFetching,
  authorSuccess,
  authorFailure,
} from '../../redux/authors'
import { ValidatedTextField, Label } from './FormItems'

const renderCheckbox = ({ input }) => (
  <Checkbox checked={input.value} type="checkbox" {...input} />
)

const AuthorEdit = ({
  isFetching,
  setAuthorEdit,
  handleSubmit,
  parseAuthorType,
  index,
  isSubmitting,
  isCorresponding,
  email,
  changeCorresponding,
  ...rest
}) => (
  <Root>
    <Header>
      <TitleContainer>
        <span>{parseAuthorType(isSubmitting, isCorresponding, index)}</span>
        {!isSubmitting && (
          <Fragment>
            <Field
              component={renderCheckbox}
              name="edit.isCorresponding"
              onChange={changeCorresponding(email)}
            />
            <label>Corresponding</label>
          </Fragment>
        )}
      </TitleContainer>

      <ButtonsContainer>
        <ClickableIcon onClick={setAuthorEdit(-1)}>
          <Icon size={3}>x-circle</Icon>
        </ClickableIcon>
        {!isFetching ? (
          <ClickableIcon onClick={handleSubmit}>
            <Icon size={3}>check-circle</Icon>
          </ClickableIcon>
        ) : (
          <Spinner size={3} />
        )}
      </ButtonsContainer>
    </Header>

    <Row>
      <ValidatedTextField
        isRequired
        label="First name*"
        name="edit.firstName"
      />
      <ValidatedTextField isRequired label="Last name*" name="edit.lastName" />
    </Row>

    <Row>
      <Label label="Email" value={email} />
      <ValidatedTextField
        isRequired
        label="Affiliation*"
        name="edit.affiliation"
      />
    </Row>
  </Root>
)

export default compose(
  connect(
    state => ({
      isFetching: getAuthorFetching(state),
    }),
    { changeForm, authorSuccess, authorFailure },
  ),
  withProps(props => ({
    initialValues: {
      edit: pick(props, [
        'id',
        'isCorresponding',
        'isSubmitting',
        'firstName',
        'lastName',
        'email',
        'affiliation',
      ]),
    },
  })),
  withHandlers({
    changeCorresponding: ({ changeForm, setAsCorresponding }) => email => (
      evt,
      newValue,
    ) => {
      setAsCorresponding(email)()
      changeForm('edit', 'edit.isCorresponding', newValue)
    },
  }),
  reduxForm({
    form: 'edit',
    onSubmit: (
      values,
      dispatch,
      { setAuthorEdit, setAuthors, authors, index, changeForm, project },
    ) => {
      const newAuthor = values.edit
      editAuthor(project.id, newAuthor.id, newAuthor).then(
        () => {
          getAuthors(project.id).then(
            data => {
              dispatch(authorSuccess())
              setAuthorEdit(-1)()
              setAuthors(data)
            },
            err => dispatch(authorFailure(err)),
          )
        },
        err => dispatch(authorFailure(err)),
      )
    },
  }),
)(AuthorEdit)

// #region styled-components
const defaultText = css`
  color: ${({ theme }) => theme.colorText};
  font-size: ${({ theme }) => theme.fontSizeBaseSmall};
  font-family: ${({ theme }) => theme.fontReading};
`

const Row = styled.div`
  display: flex;
  flex-direction: row;
  margin: 10px 0;
`

const TitleContainer = styled.div`
  align-items: center;
  display: flex;
  flex-direction: row;

  > span {
    ${defaultText};
    font-size: ${({ theme }) => theme.fontSizeHeading6};
    margin-right: 10px;
    font-weight: 600;
    text-align: left;
  }

  label {
    ${defaultText};
    text-align: left;
  }
`

const ButtonsContainer = styled.div`
  align-items: center;
  display: flex;
  flex-direction: row;
  justify-content: flex-end;
`

const ClickableIcon = styled.div`
  align-items: center;
  cursor: pointer;
  display: flex;
  flex: 1;
  flex-direction: column;
  justify-content: center;
  margin: 0 12px;
`

const Header = styled.div`
  align-items: center;
  display: flex;
  flex-direction: row;
  justify-content: space-between;
`

const Root = styled.div`
  border: ${({ theme }) => theme.borderDefault};
  margin: 10px 0;
  padding: 10px;
  display: flex;
  flex-direction: column;
`
// #endregion
